# -*- coding: utf-8 -*-

import datetime
import mysql.simplesql as sql
from nes_common.nes_types import toTime
from objects.extend_objects.additionalprice import getEffectiveDate, insertTprices

tables = ['tmc', 'tprices', 'tmcsalerestrict', 'barcodes']


def barcodes(inventItem):
    return inventItem.get('barcodes', [])


def additionalPrices(inventOrBarcode):
    return inventOrBarcode.get('additionalprices', [])


def timeRestrictions(inventOrBarcode):
    return inventOrBarcode.get('sellrestrictperiods', [])


def quantModeForOptions(options):
    quantMode = 0
    if options.get("quantityoptions", {}).get("enabledefaultquantity", 1) == 1:
        quantMode |= 1
    if options.get("quantityoptions", {}).get("enablequantitymanual", 1) == 1:
        quantMode |= 2
    if options.get("quantityoptions", {}).get("enablequantitybarcode", 1) == 1:
        quantMode |= 4
    if options.get("quantityoptions", {}).get("enablequantityscales", 1) == 1:
        quantMode |= 8
    if options.get("quantityoptions", {}).get("enablequantitylimit", 0) == 1:
        quantMode |= 16
    if options.get("quantityoptions", {}).get("requirequantitymanual", 0) == 1:
        quantMode |= 32
    if options.get("quantityoptions", {}).get("requirequantityscales", 0) == 1:
        quantMode |= 64
    if options.get("quantityoptions", {}).get("requirequantitybarcode", 0) == 1:
        quantMode |= 128
    if options.get("quantityoptions", {}).get("enabledocumentquantitylimit", 0) == 1:
        quantMode |= 256
    if options.get("quantityoptions", {}).get("enabledocumentquantitylimit", 0) == 2:
        quantMode |= 512
    if options.get("quantityoptions", {}).get("autogetquantityfromscales", 0) == 1:
        quantMode |= 1024
    return quantMode


def bcodeModeForOptions(options):
    bcodeMode = 0
    if options.get("inventitemoptions", {}).get("enablebarcodemanual", 1) == 1:
        bcodeMode |= 1
    if options.get("inventitemoptions", {}).get("enablebarcodescanner", 1) == 1:
        bcodeMode |= 2
    return bcodeMode


def opModeForOptions(options):
    opMode = 0
    if options.get("inventitemoptions", {}).get("disableinventsale", 0) == 1:
        opMode |= 1
    if options.get("inventitemoptions", {}).get("disableinventback", 0) == 1:
        opMode |= 2
    if options.get("inventitemoptions", {}).get("disableinventshow", 0) == 1:
        opMode |= 4
    if options.get("inventitemoptions", {}).get("disablebackinsale", 0) == 1:
        opMode |= 8
    if options.get("inventitemoptions", {}).get("visualverify", 0) == 1:
        opMode |= 16
    if options.get("inventitemoptions", {}).get("ageverify", 0) == 1:
        opMode |= 32
    if options.get("inventitemoptions", {}).get("requiresalerestrict", 0) == 1:
        opMode |= 64
    if options.get("inventitemoptions", {}).get("egaisverify", 0) == 1:
        opMode |= 128
    if options.get("inventitemoptions", {}).get("prepackaged", 0) == 1:
        opMode |= 256
    if options.get("inventitemoptions", {}).get("nopdfegaisverify", 0) == 1:
        opMode |= 512
    if options.get("inventitemoptions", {}).get("alcoset", 0) == 1:
        opMode |= 1024
    if options.get("inventitemoptions", {}).get("freesale", 0) == 1:
        opMode |= 2048
    if options.get("inventitemoptions", {}).get("rfidverify", 0) == 1:
        opMode |= 4096
    if options.get("inventitemoptions", {}).get("weightcontrolbypass", 0) == 1:
        opMode |= 8192
    if options.get("inventitemoptions", {}).get("lowweight", 0) == 1:
        opMode |= 16384
    if options.get("inventitemoptions", {}).get("tobacco", 0) == 1:
        opMode |= 32768
    if options.get("inventitemoptions", {}).get("shoes", 0) == 1:
        opMode |= 65536
    if options.get("inventitemoptions", {}).get("fuzzyweight", 0) == 1:
        opMode |= 131072
    if options.get("inventitemoptions", {}).get("ignoremarking", 0) == 1:
        opMode |= 262144
    if options.get("inventitemoptions", {}).get("markdownverify", 0) == 1:
        opMode |= 524288
    return opMode


def deptModeForOptions(options):
    deptMode = 0
    if options.get("inventitemoptions", {}).get("enabledepartmentmanual", 1) == 1:
        deptMode |= 1
    if options.get("inventitemoptions", {}).get("requiredepartmentmanual", 0) == 1:
        deptMode |= 2
    return deptMode


def priceModeForOptions(options):
    priceMode = 0
    if options.get("priceoptions", {}).get("enablepricemanual", 1) == 1:
        priceMode |= 1
    if options.get("priceoptions", {}).get("requirepricemanual", 0) == 1:
        priceMode |= 2
    if options.get("priceoptions", {}).get("requireselectprice", 0) == 1:
        priceMode |= 4
    if options.get("priceoptions", {}).get("requiredeferredprice", 0) == 1:
        priceMode |= 8
    if options.get("priceoptions", {}).get("enableexcisemarkprice", 0) == 1:
        priceMode |= 16
    return priceMode


def extractTmcPropertiesFromInventItem(inventItem):
    vatcodes = inventItem.get('lvatcode', [0, 0, 0, 0, 0])
    taxGroupCode = inventItem.get('taxgroupcode', None)
    vatcodes += [0] * (5 - len(vatcodes))

    if taxGroupCode:
        for n in range(5):
            vatcodes[n] = int(taxGroupCode) * 100 + n + 1

    tmc = sql.Insert(table='tmc', values={
        'code': inventItem.get('inventcode', None),
        'bcode': inventItem.get('inventcode', None),
        'vatcode1': vatcodes[0],
        'vatcode2': vatcodes[1],
        'vatcode3': vatcodes[2],
        'vatcode4': vatcodes[3],
        'vatcode5': vatcodes[4],
        'dcode': inventItem.get('deptcode', 0),
        'name': inventItem.get('name', ''),
        'articul': inventItem.get('articul', ''),
        'cquant': inventItem.get('cquant', 1.0),
        'measure': inventItem.get('measurecode', 0),
        'pricetype': inventItem.get('discautoscheme', 0),
        'price': inventItem.get('price', 0.0),
        'minprice': inventItem.get('minprice', 0.0),
        'valcode': 0,
        'quantdefault': inventItem.get('defaultquantity', 1.0),
        'quantlimit': inventItem.get('options', {}).get('quantityoptions', {}).get('quantitylimit', 0),
        'ostat': 0,
        'links': 0,
        'quant_mode': quantModeForOptions(inventItem.get('options', {})),
        'bcode_mode': bcodeModeForOptions(inventItem.get('options', {})),
        'op_mode': opModeForOptions(inventItem.get('options', {})) if inventItem.get('options', {}).get('inventitemoptions', {}) else inventItem.get('opmode', 0),
        'dept_mode': deptModeForOptions(inventItem.get('options', {})),
        'price_mode': priceModeForOptions(inventItem.get('options', {})),
        'groupcode': inventItem.get('inventgroup', None),
        'extendetoptions': inventItem.get('extendedoptions', None),
        'tara_mode': inventItem.get('taramode', None),
        'tara_default': '0',
        'unit_weight': inventItem.get('taracapacity', None),
        'remain': inventItem.get('remain', None),
        'aspectschemecode': inventItem.get('aspectschemecode', None),
        'aspectvaluesetcode': inventItem.get('aspectvaluesetcode', None),
        'aspectusecase': inventItem.get('aspectusecase', None),
        'aspectselectionrule': inventItem.get('aspectselectionrule', None),
        'remaindate': inventItem.get('remaindate', None) if inventItem.get('remaindate', None) else None,
        'documentquantlimit': inventItem.get('options', {}).get('quantityoptions', {}).get('documentquantlimit', 0),
        'age': inventItem.get('age', None),
        'alcoholpercent': inventItem.get('alcoholpercent', None),
        'inn': inventItem.get('inn', None),
        'kpp': inventItem.get('kpp', None),
        'alctypecode': inventItem.get('alctypecode', 0),
        'manufacturercountrycode': inventItem.get('manufacturercountrycode', None),
        'paymentobject': inventItem.get('paymentobject', None),
        'loyaltymode': inventItem.get('loyaltymode', None),
        'minretailprice': inventItem.get('minretailprice', None),
        'ntin': inventItem.get('ntin', None),
        'packagecode': inventItem.get('packagecode', None),
        'ownertype': inventItem.get('ownertype', None),
        'tara_flag': inventItem.get('taraflag', None),
    })
    return tmc


def extractBarcodesFromInventItem(inventItem):
    return sql.Insert(table='barcodes', values={
        'code': inventItem.get('inventcode'),
        'barcode': inventItem.get('barcode'),
        'name': inventItem.get('name', None),
        'price': inventItem.get('price', None),
        'cquant': inventItem.get('cquant', None),
        'measure': inventItem.get('measurecode', None),
        'quantdefault': inventItem.get('quantdefault', 1.0),
        'aspectvaluesetcode': inventItem.get('aspectvaluesetcode', None),
        'packingmeasure': inventItem.get('packingmeasure', None),
        'packingprice': inventItem.get('packingprice', None),
        'minprice': inventItem.get('minprice', None),
        'minretailprice': inventItem.get('minretailprice', None),
        'customsdeclarationnumber': inventItem.get('customsdeclarationnumber', None),
        'tmctype': inventItem.get('tmctype', None),
        'ntin': inventItem.get('ntin', None),
        'packagecode': inventItem.get('packagecode', None),
    })


def tmcWithOwnBarcode(inventItem):
    barcode = inventItem.get('barcode', None)
    return extractBarcodesFromInventItem(inventItem) if barcode is not None and barcode.strip() != '' else None


def propagateBarcode(inventItem, barcodeItem):
    result = inventItem.copy()
    for key in barcodeItem.keys():
        result[key] = barcodeItem[key]
    return result


def tmcWithBarcodes(inventItem):
    for bcode in barcodes(inventItem):
        mixedInvent = propagateBarcode(inventItem, bcode)
        tmc = tmcWithOwnBarcode(mixedInvent)
        if tmc:
            yield tmc


def tpricesForInventItem(inventItem):
    tmpBcodes = []
    for bcode in [inventItem.get('inventcode', None), inventItem.get('barcode', None)]:
        if bcode not in tmpBcodes:
            tmpBcodes.append(bcode)
            if bcode is None:
                continue
            for additionalPrice in additionalPrices(inventItem):
                index = additionalPrice.get('pricecode', None)
                if index is None:
                    continue
                additionalPrice = additionalPrice.copy()
                additionalPrice['barcode'] = bcode
                for query in insertTprices(additionalPrice):
                    yield query


def tpricesForBarcode(inventItem, barcode):
    for additionalPrice in additionalPrices(barcode):
        index = additionalPrice.get('pricecode', None)
        bcode = barcode.get('barcode', None)
        tprices = sql.Insert(table='tprices', values={
            'tindex': index,
            'bcode': bcode,
            'price': additionalPrice.get('price', 0.0),
            'name': additionalPrice.get('name', ''),
            'documentid': additionalPrice.get('documentid', ''),
            'effectivedate': getEffectiveDate(additionalPrice),
            'packingprice': additionalPrice.get('packingprice', None)
        })
        if index and bcode:
            yield tprices


def tpricesForBarcodes(inventItem):
    for barcode in barcodes(inventItem):
        tpricesIndexes = list(map(lambda tprice: tprice.get('pricecode', 0), additionalPrices(barcode)))
        yield sql.DeleteNotNestedItems(table='tprices', mainCodeField='bcode',
                mainCodeValue=barcode.get('barcode', ''), nestedCodeField='tindex', nestedCodeIn=tpricesIndexes)
        for t in tpricesForBarcode(inventItem, barcode):
            yield t


def timeRestrictionsForItem(inventItemOrBarcode):
    for r in timeRestrictions(inventItemOrBarcode):
        yield sql.Insert(table='tmcsalerestrict', values={
            'dateend': r.get('dateend', None) if r.get('dateend', None) else None,
            'datebeg': r.get('datestart', None) if r.get('datestart', None) else None,
            'daysend': r.get('dayend', None),
            'daysbeg': r.get('daystart', None),
            'timeend': toTime(r.get('timeend', None)),
            'timebeg': toTime(r.get('timestart', None))
        })


def timeRestrictionsForInventItem(inventItem):
    zeroTime = datetime.time(0, 0, 0)  # нужно для одинакового формирования salerestrictcode, т.к. в python2 время (0,0,0) считается как None
    for r in timeRestrictionsForItem(inventItem):
        hourForKey = r['timeend'] if r['timebeg'] is None or r['timebeg'] == zeroTime else r['timebeg']
        dayForKey = ""
        if r['daysbeg']:
            dayForKey = str(r['daysbeg'])
        elif r['daysend']:
            dayForKey = str(r['daysend'])

        r['tmccode'] = inventItem.get('inventcode', None)
        r['salerestrictcode'] = inventItem.get('inventcode', '') + str(hourForKey.hour) + dayForKey
        yield r


def timeRestrictionsForBarcode(inventItem, barcode):
    return []
#    for r in timeRestrictionsForItem(inventItem):
#        r['tmccode'] = inventItem.get('inventcode', None)
#        yield r


def timeRestrictionsForBarcodes(inventItem):
    for barcode in barcodes(inventItem):
        for r in timeRestrictionsForBarcode(inventItem, barcode):
            yield r


def insertInventItem(inventItem):
    t = extractTmcPropertiesFromInventItem(inventItem)
    if t:
        yield t
    bcodes = list(map(lambda barcode: barcode.get('barcode'), barcodes(inventItem)))
    # Здесь появляется флаг "штрих-кода нет среди штрих-кодов товара" связано это с тем, что если в справочнике в
    # barcode у товара и в секции barcodes будет один и тот же ШК, он будет загружаться в БД дважды: сначала в
    # barcodes, затем в tmc, в итоге затираются поля barcodes, которых нет в tmc. Теперь, если ШК из секции barcodes
    # совпадает с barcode(т.е. barcodeNotInInventBcodes = False), он будет прогружен только в barcodes
    barcodeNotInInventBcodes = False
    if inventItem.get("barcode") and inventItem.get("barcode") not in bcodes:
        bcodes.append(inventItem.get("barcode"))
        barcodeNotInInventBcodes = True
    yield sql.DeleteNotNestedItems(table='barcodes', mainCodeField='code',
            mainCodeValue=inventItem.get('inventcode', ''), nestedCodeField='barcode', nestedCodeIn=bcodes)
    tpricesIndexes = list(map(lambda tprice: tprice.get('pricecode', 0), additionalPrices(inventItem)))
    yield sql.DeleteNotNestedItems(table='tprices', mainCodeField='bcode',
            mainCodeValue=inventItem.get('inventcode', ''), nestedCodeField='tindex', nestedCodeIn=tpricesIndexes)
    for t in tmcWithBarcodes(inventItem):
        yield t
    if barcodeNotInInventBcodes:
        t = tmcWithOwnBarcode(inventItem)
        if t:
            yield sql.DeleteNotNestedItems(table='tprices', mainCodeField='bcode',
                mainCodeValue=inventItem.get('barcode', ''), nestedCodeField='tindex', nestedCodeIn=tpricesIndexes)
            yield t

    for p in tpricesForInventItem(inventItem):
        yield p
    for p in tpricesForBarcodes(inventItem):
        yield p
    yield sql.Delete(table='tmcsalerestrict', whereFields=['tmccode'], whereValues=[inventItem.get('inventcode', '')])
    for r in timeRestrictionsForInventItem(inventItem):
        yield r
    for r in timeRestrictionsForBarcodes(inventItem):
        yield r


def deleteInventItemByCode(code):
    barcodes = sql.Select(table='tmc', fields=['bcode'], whereFields=['code'], whereValues=[code])
    delbarcodes = sql.Select(table='barcodes', fields=['barcode'], whereFields=['code'], whereValues=[code])
    yield sql.Delete(table='tmcsalerestrict', whereFields=['tmccode'], whereValues=[code])
    yield sql.Delete(table='tprices', whereFields=['bcode'], whereValues=delbarcodes)
    yield sql.Delete(table='tprices', whereFields=['bcode'], whereValues=barcodes)
    yield sql.Delete(table='tprices', whereFields=['bcode'], whereValues=[code])
    yield sql.Delete(table='barcodes', whereFields=['code'], whereValues=[code])
    yield sql.Delete(table='tmc', whereFields=['code'], whereValues=[code])


def deleteInventItem(inventItem):
    code = inventItem.get('inventcode', None)
    barcodes = filter(lambda x: x is not None, map(lambda x: x.get('barcode', None), inventItem.get('barcodes', [])))
    for k in deleteInventItemByCode(code):
        yield k
    # TODO: НУЖНО ЕЩЁ ИЗ ТОВАРА ШК ГРОХНУТЬ!!!
    for barcode in barcodes:
        yield sql.Delete(table='tmc', whereFields=['bcode'], whereValues=[barcode])


def clearInventory():
    yield sql.DeleteAll(table='tmc')
    yield sql.DeleteAll(table='tmcsalerestrict')
    yield sql.DeleteAll(table='tprices')
    yield sql.DeleteAll(table='barcodes')


def updateInventItem(inventItem):
    fields = []
    values = []

    if inventItem.get('aspectusecase'):
        fields.append('aspectusecase')
        values.append(inventItem.get('aspectusecase'))
    if inventItem.get('taramode'):
        fields.append('tara_mode')
        values.append(inventItem.get('taramode'))
    if inventItem.get('taracapacity'):
        fields.append('unit_weight')
        values.append(inventItem.get('taracapacity'))
    if inventItem.get('options'):
        fields.append('quant_mode')
        values.append(quantModeForOptions(inventItem.get('options', {})))
        if inventItem.get('options', {}).get('quantityoptions', {}).get('quantitylimit') is not None:
            fields.append('quantlimit')
            values.append(inventItem.get('options', {}).get('quantityoptions', {}).get('quantitylimit', 0))
        fields.append('links')
        values.append(0)
        fields.append('bcode_mode')
        values.append(bcodeModeForOptions(inventItem.get('options', {})))
        fields.append('op_mode')
        values.append(opModeForOptions(inventItem.get('options', {})))
        fields.append('dept_mode')
        values.append(deptModeForOptions(inventItem.get('options', {})))
        fields.append('price_mode')
        values.append(priceModeForOptions(inventItem.get('options', {})))
        if inventItem.get('options', {}).get('quantityoptions', {}).get('documentquantlimit') is not None:
            fields.append('documentquantlimit')
            values.append(inventItem.get('options', {}).get('quantityoptions', {}).get('documentquantlimit', 0))
    if inventItem.get('inn'):
        fields.append('inn')
        values.append(inventItem.get('inn'))
    if inventItem.get('kpp'):
        fields.append('kpp')
        values.append(inventItem.get('kpp'))
    if inventItem.get('minretailprice'):
        fields.append('minretailprice')
        values.append(inventItem.get('minretailprice'))
    if inventItem.get('alcoholpercent'):
        fields.append('alcoholpercent')
        values.append(inventItem.get('alcoholpercent'))
    if inventItem.get('alctypecode'):
        fields.append('alctypecode')
        values.append(inventItem.get('alctypecode'))
    if inventItem.get('extendedoptions'):
        fields.append('extendetoptions')
        values.append(inventItem.get('extendedoptions'))
    if inventItem.get('sellrestrictperiods'):
        r = inventItem.get('sellrestrictperiods')
        hourForKey = r.get('timestart', None) if r.get('timestart', None) else r.get('timeend', None)
        yield sql.Insert(table='tmcsalerestrict', values={
            'salerestrictcode': inventItem.get('inventcode', '') + str(toTime(hourForKey).hour),
            'dateend': r.get('dateend', None) if r.get('dateend', None) else None,
            'datebeg': r.get('datestart', None) if r.get('datestart', None) else None,
            'daysend': r.get('dayend', None),
            'daysbeg': r.get('daystart', None),
            'timeend': r.get('timeend', None) if r.get('timeend', None) else None,
            'timebeg': r.get('timestart', None) if r.get('timestart', None) else None,
            'tmccode': inventItem.get('inventcode', None)
        })
    if inventItem.get('aspectremains'):
        r = inventItem.get('aspectremains')

        yield sql.Update(table='tmc', fields=['aspectschemecode'], values=[r.get('aspectschemecode', None)],
                whereField=['code'], whereValue=[inventItem.get('inventcode', 0)])
        yield sql.Update(table='aspectvalueset', fields=['price', 'remain'],
                values=[r.get('price', None), r.get('remain', None)], whereField=['aspectvaluesetcode'],
                whereValue=[r.get('aspectvaluesetcode', None)])

    if fields:
        yield sql.Update(table='tmc', fields=fields, values=values,
                whereField=['bcode'], whereValue=[inventItem.get('inventcode', 0)])
