# -*- coding: UTF-8 -*-

import fcntl
import logging
import os
import ujson


class ErrorHandler(logging.Handler):
    """
    Хэндлер для логгера.
    Срабатывает при логировании ошибок(logger.error(...)) и увеличивает их счётчик в переданном в конструкторе файле.
    Также контролирует наличие директории и файла с ошибками.
    При записи в файл блокирует его.
    """
    def __init__(self, filePath):
        """
        :param str filePath - путь до файла с ошибками
        """
        super(ErrorHandler, self).__init__(logging.ERROR)
        self.filePath = filePath

    def emit(self, record):
        """ Переопределённый метод базового класса, который вызывается при логировании сообщения
        :param LogRecord record - объект для записи в лог
        """
        ErrorHandler.__increaseErrorsCount(self.filePath)

    @staticmethod
    def __createDirectory(filePath):
        """ Создание директории по пути файла с ошибками
        :param str filePath - путь до файла с ошибками
        """
        if not os.path.exists(os.path.dirname(filePath)):
            os.makedirs(os.path.dirname(filePath))

    @staticmethod
    def __increaseErrorsCount(filePath):
        """ Увеличение счётчика ошибок в файле с ошибками. При записи используется блокировка файла
        :param str filePath - путь до файла с ошибками
        """
        ErrorHandler.__createDirectory(filePath)
        data = dict()
        f = -1
        try:
            f = os.open(filePath, os.O_RDWR)
        except:
            pass
        if f == -1:
            return
        if fcntl.flock(f, fcntl.LOCK_EX):
            os.close(f)
            return
        buf = ''
        while True:
            a = os.read(f, 256)
            if not a:
                break
            buf += a
        try:
            data = ujson.loads(buf)
        except:
            pass

        data['errorsCount'] = data.get('errorsCount', 0) + 1

        try:
            os.ftruncate(f, 0)
            os.lseek(f, 0, os.SEEK_SET)
            os.write(f, ujson.dumps(data).encode())
        except:
            pass
        fcntl.flock(f, fcntl.LOCK_UN)
        os.close(f)
